import tkinter as Tk
import time
import random

class Ball:
    def __init__(self, cvs, paddle):
        # Ball の画像を作り、キャンバスに突っ込んだりする
        self.canvas = cvs
        self.id = cvs.create_oval(10, 10, 25, 25, fill="red")
        self.canvas.move(self.id, 245, 100)
        self.paddle = paddle
        self.vx = random.choice([-3, -2, -1, 1, 2, 3])
        self.vy = -3
        self.canvas_width = self.canvas.winfo_width()
        self.canvas_height = self.canvas.winfo_height()
        self.is_fallen = False
    
    def update(self):
        # Ball を移動したりする
        self.canvas.move(self.id, self.vx, self.vy)
        self.solve_collision()

    def is_paddle_hit(self):
        # パドルが当たっていれば True
        paddle_pos = self.canvas.coords(self.paddle.id)
        ball_pos = self.canvas.coords(self.id)
        if paddle_pos[0] <= ball_pos[2] and ball_pos[0] <= paddle_pos[2] \
           and paddle_pos[1] <= ball_pos[3] <= paddle_pos[3]:
            return True
        return False
    
    def solve_collision(self):
        # Ball の衝突まわりを処理する
        # まずは Ball の座標取得
        pos = self.canvas.coords(self.id)
        if pos[0] < 0:
            # 左壁に衝突
            self.vx *= -1
        elif pos[2] > self.canvas_width:
            # 右壁に衝突
            self.vx *= -1
        if pos[1] < 0:
            # 天井に衝突
            self.vy *= -1
        elif self.is_paddle_hit():
            # パドルに衝突!
            self.vy *= -1
        elif pos[3] > self.canvas_height:
            # 床に衝突
            self.is_fallen = True
        

class Paddle:
    def __init__(self, cvs):
        # Paddle の画像を作り、キャンバスに突っ込んだりする
        self.canvas = cvs
        self.id = cvs.create_rectangle(0, 0, 100, 10, fill="blue")
        self.canvas.move(self.id, 200, 300)
        self.vx = 0
        self.canvas_width = self.canvas.winfo_width()

        self.canvas.bind_all("<KeyPress-Left>", self.turn)
        self.canvas.bind_all("<KeyPress-Right>", self.turn)
        
    def update(self):
        # 速度を基に移動したりする
        self.canvas.move(self.id, self.vx, 0)
        self.solve_collision()
    
    def solve_collision(self):
        # 衝突していたら相応の処理をする
        pos = self.canvas.coords(self.id)
        if pos[0] < 0:
            # 左壁に衝突
            self.vx = 0
        elif pos[2] > self.canvas_width:
            # 右壁に衝突
            self.vx = 0
    
    def turn(self, ev):
        # キー入力を受け付ける
        if ev.keycode == 37:
            # [←] キー (keycode:37)
            self.vx = -4
        elif ev.keycode == 39:
            # [→] キー (keycode:39)
            self.vx = 4

class GameOverSprite:
    def __init__(self, cvs):
        # キャンバスにテキストを「非表示で」突っ込んでおく
        self.canvas = cvs
        self.id = cvs.create_text(240, 64,
                                  text='GAME OVER!',
                                  fill='red',
                                  font=('Courier', 24),
                                  state='hidden')
        
    def show(self):
        # 非表示で突っ込んであるテキストを表示する
        self.canvas.itemconfigure(self.id, state='normal')
        

class Game:
    def __init__(self):
        self.wnd = Tk.Tk()
        self.wnd.title("BOUNCE!")
        self.wnd.resizable(0, 0)
        self.cvs = Tk.Canvas(self.wnd, width=480, height=480)
        self.cvs.pack()
        self.wnd.update()

        self.paddle = Paddle(self.cvs)
        self.ball = Ball(self.cvs, self.paddle)
        self.gameOverSprite = GameOverSprite(self.cvs)

    def update(self):
        # ここにゲームを制御する処理
        if not self.ball.is_fallen:
            self.ball.update()
            self.paddle.update()
        else:
            self.gameOverSprite.show()
    
    def run(self):
        while True:
            self.update()
            self.wnd.update_idletasks()
            self.wnd.update()
            time.sleep(0.03)

g = Game()
g.run()


